##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class ProfileViewBasic(base.ModelBase):
    """Definition model for :obj:`app.bsky.actor.defs`."""

    did: string_formats.Did  #: Did.
    handle: string_formats.Handle  #: Handle.
    associated: t.Optional['models.AppBskyActorDefs.ProfileAssociated'] = None  #: Associated.
    avatar: t.Optional[string_formats.Uri] = None  #: Avatar.
    created_at: t.Optional[string_formats.DateTime] = None  #: Created at.
    display_name: t.Optional[str] = Field(default=None, max_length=640)  #: Display name.
    labels: t.Optional[t.List['models.ComAtprotoLabelDefs.Label']] = None  #: Labels.
    verification: t.Optional['models.AppBskyActorDefs.VerificationState'] = None  #: Verification.
    viewer: t.Optional['models.AppBskyActorDefs.ViewerState'] = None  #: Viewer.

    py_type: t.Literal['app.bsky.actor.defs#profileViewBasic'] = Field(
        default='app.bsky.actor.defs#profileViewBasic', alias='$type', frozen=True
    )


class ProfileView(base.ModelBase):
    """Definition model for :obj:`app.bsky.actor.defs`."""

    did: string_formats.Did  #: Did.
    handle: string_formats.Handle  #: Handle.
    associated: t.Optional['models.AppBskyActorDefs.ProfileAssociated'] = None  #: Associated.
    avatar: t.Optional[string_formats.Uri] = None  #: Avatar.
    created_at: t.Optional[string_formats.DateTime] = None  #: Created at.
    description: t.Optional[str] = Field(default=None, max_length=2560)  #: Description.
    display_name: t.Optional[str] = Field(default=None, max_length=640)  #: Display name.
    indexed_at: t.Optional[string_formats.DateTime] = None  #: Indexed at.
    labels: t.Optional[t.List['models.ComAtprotoLabelDefs.Label']] = None  #: Labels.
    verification: t.Optional['models.AppBskyActorDefs.VerificationState'] = None  #: Verification.
    viewer: t.Optional['models.AppBskyActorDefs.ViewerState'] = None  #: Viewer.

    py_type: t.Literal['app.bsky.actor.defs#profileView'] = Field(
        default='app.bsky.actor.defs#profileView', alias='$type', frozen=True
    )


class ProfileViewDetailed(base.ModelBase):
    """Definition model for :obj:`app.bsky.actor.defs`."""

    did: string_formats.Did  #: Did.
    handle: string_formats.Handle  #: Handle.
    associated: t.Optional['models.AppBskyActorDefs.ProfileAssociated'] = None  #: Associated.
    avatar: t.Optional[string_formats.Uri] = None  #: Avatar.
    banner: t.Optional[string_formats.Uri] = None  #: Banner.
    created_at: t.Optional[string_formats.DateTime] = None  #: Created at.
    description: t.Optional[str] = Field(default=None, max_length=2560)  #: Description.
    display_name: t.Optional[str] = Field(default=None, max_length=640)  #: Display name.
    followers_count: t.Optional[int] = None  #: Followers count.
    follows_count: t.Optional[int] = None  #: Follows count.
    indexed_at: t.Optional[string_formats.DateTime] = None  #: Indexed at.
    joined_via_starter_pack: t.Optional['models.AppBskyGraphDefs.StarterPackViewBasic'] = (
        None  #: Joined via starter pack.
    )
    labels: t.Optional[t.List['models.ComAtprotoLabelDefs.Label']] = None  #: Labels.
    pinned_post: t.Optional['models.ComAtprotoRepoStrongRef.Main'] = None  #: Pinned post.
    posts_count: t.Optional[int] = None  #: Posts count.
    verification: t.Optional['models.AppBskyActorDefs.VerificationState'] = None  #: Verification.
    viewer: t.Optional['models.AppBskyActorDefs.ViewerState'] = None  #: Viewer.

    py_type: t.Literal['app.bsky.actor.defs#profileViewDetailed'] = Field(
        default='app.bsky.actor.defs#profileViewDetailed', alias='$type', frozen=True
    )


class ProfileAssociated(base.ModelBase):
    """Definition model for :obj:`app.bsky.actor.defs`."""

    chat: t.Optional['models.AppBskyActorDefs.ProfileAssociatedChat'] = None  #: Chat.
    feedgens: t.Optional[int] = None  #: Feedgens.
    labeler: t.Optional[bool] = None  #: Labeler.
    lists: t.Optional[int] = None  #: Lists.
    starter_packs: t.Optional[int] = None  #: Starter packs.

    py_type: t.Literal['app.bsky.actor.defs#profileAssociated'] = Field(
        default='app.bsky.actor.defs#profileAssociated', alias='$type', frozen=True
    )


class ProfileAssociatedChat(base.ModelBase):
    """Definition model for :obj:`app.bsky.actor.defs`."""

    allow_incoming: t.Union[t.Literal['all'], t.Literal['none'], t.Literal['following'], str]  #: Allow incoming.

    py_type: t.Literal['app.bsky.actor.defs#profileAssociatedChat'] = Field(
        default='app.bsky.actor.defs#profileAssociatedChat', alias='$type', frozen=True
    )


class ViewerState(base.ModelBase):
    """Definition model for :obj:`app.bsky.actor.defs`. Metadata about the requesting account's relationship with the subject account. Only has meaningful content for authed requests."""

    blocked_by: t.Optional[bool] = None  #: Blocked by.
    blocking: t.Optional[string_formats.AtUri] = None  #: Blocking.
    blocking_by_list: t.Optional['models.AppBskyGraphDefs.ListViewBasic'] = None  #: Blocking by list.
    followed_by: t.Optional[string_formats.AtUri] = None  #: Followed by.
    following: t.Optional[string_formats.AtUri] = None  #: Following.
    known_followers: t.Optional['models.AppBskyActorDefs.KnownFollowers'] = None  #: Known followers.
    muted: t.Optional[bool] = None  #: Muted.
    muted_by_list: t.Optional['models.AppBskyGraphDefs.ListViewBasic'] = None  #: Muted by list.

    py_type: t.Literal['app.bsky.actor.defs#viewerState'] = Field(
        default='app.bsky.actor.defs#viewerState', alias='$type', frozen=True
    )


class KnownFollowers(base.ModelBase):
    """Definition model for :obj:`app.bsky.actor.defs`. The subject's followers whom you also follow."""

    count: int  #: Count.
    followers: t.List['models.AppBskyActorDefs.ProfileViewBasic'] = Field(min_length=0, max_length=5)  #: Followers.

    py_type: t.Literal['app.bsky.actor.defs#knownFollowers'] = Field(
        default='app.bsky.actor.defs#knownFollowers', alias='$type', frozen=True
    )


class VerificationState(base.ModelBase):
    """Definition model for :obj:`app.bsky.actor.defs`. Represents the verification information about the user this object is attached to."""

    trusted_verifier_status: t.Union[
        t.Literal['valid'], t.Literal['invalid'], t.Literal['none'], str
    ]  #: The user's status as a trusted verifier.
    verifications: t.List[
        'models.AppBskyActorDefs.VerificationView'
    ]  #: All verifications issued by trusted verifiers on behalf of this user. Verifications by untrusted verifiers are not included.
    verified_status: t.Union[
        t.Literal['valid'], t.Literal['invalid'], t.Literal['none'], str
    ]  #: The user's status as a verified account.

    py_type: t.Literal['app.bsky.actor.defs#verificationState'] = Field(
        default='app.bsky.actor.defs#verificationState', alias='$type', frozen=True
    )


class VerificationView(base.ModelBase):
    """Definition model for :obj:`app.bsky.actor.defs`. An individual verification for an associated subject."""

    created_at: string_formats.DateTime  #: Timestamp when the verification was created.
    is_valid: bool  #: True if the verification passes validation, otherwise false.
    issuer: string_formats.Did  #: The user who issued this verification.
    uri: string_formats.AtUri  #: The AT-URI of the verification record.

    py_type: t.Literal['app.bsky.actor.defs#verificationView'] = Field(
        default='app.bsky.actor.defs#verificationView', alias='$type', frozen=True
    )


Preferences = t.List[
    te.Annotated[
        t.Union[
            'models.AppBskyActorDefs.AdultContentPref',
            'models.AppBskyActorDefs.ContentLabelPref',
            'models.AppBskyActorDefs.SavedFeedsPref',
            'models.AppBskyActorDefs.SavedFeedsPrefV2',
            'models.AppBskyActorDefs.PersonalDetailsPref',
            'models.AppBskyActorDefs.FeedViewPref',
            'models.AppBskyActorDefs.ThreadViewPref',
            'models.AppBskyActorDefs.InterestsPref',
            'models.AppBskyActorDefs.MutedWordsPref',
            'models.AppBskyActorDefs.HiddenPostsPref',
            'models.AppBskyActorDefs.BskyAppStatePref',
            'models.AppBskyActorDefs.LabelersPref',
            'models.AppBskyActorDefs.PostInteractionSettingsPref',
            'models.AppBskyActorDefs.VerificationPrefs',
        ],
        Field(discriminator='py_type'),
    ]
]


class AdultContentPref(base.ModelBase):
    """Definition model for :obj:`app.bsky.actor.defs`."""

    enabled: bool = False  #: Enabled.

    py_type: t.Literal['app.bsky.actor.defs#adultContentPref'] = Field(
        default='app.bsky.actor.defs#adultContentPref', alias='$type', frozen=True
    )


class ContentLabelPref(base.ModelBase):
    """Definition model for :obj:`app.bsky.actor.defs`."""

    label: str  #: Label.
    visibility: t.Union[
        t.Literal['ignore'], t.Literal['show'], t.Literal['warn'], t.Literal['hide'], str
    ]  #: Visibility.
    labeler_did: t.Optional[string_formats.Did] = (
        None  #: Which labeler does this preference apply to? If undefined, applies globally.
    )

    py_type: t.Literal['app.bsky.actor.defs#contentLabelPref'] = Field(
        default='app.bsky.actor.defs#contentLabelPref', alias='$type', frozen=True
    )


class SavedFeed(base.ModelBase):
    """Definition model for :obj:`app.bsky.actor.defs`."""

    id: str  #: Id.
    pinned: bool  #: Pinned.
    type: t.Union[t.Literal['feed'], t.Literal['list'], t.Literal['timeline'], str]  #: Type.
    value: str  #: Value.

    py_type: t.Literal['app.bsky.actor.defs#savedFeed'] = Field(
        default='app.bsky.actor.defs#savedFeed', alias='$type', frozen=True
    )


class SavedFeedsPrefV2(base.ModelBase):
    """Definition model for :obj:`app.bsky.actor.defs`."""

    items: t.List['models.AppBskyActorDefs.SavedFeed']  #: Items.

    py_type: t.Literal['app.bsky.actor.defs#savedFeedsPrefV2'] = Field(
        default='app.bsky.actor.defs#savedFeedsPrefV2', alias='$type', frozen=True
    )


class SavedFeedsPref(base.ModelBase):
    """Definition model for :obj:`app.bsky.actor.defs`."""

    pinned: t.List[string_formats.AtUri]  #: Pinned.
    saved: t.List[string_formats.AtUri]  #: Saved.
    timeline_index: t.Optional[int] = None  #: Timeline index.

    py_type: t.Literal['app.bsky.actor.defs#savedFeedsPref'] = Field(
        default='app.bsky.actor.defs#savedFeedsPref', alias='$type', frozen=True
    )


class PersonalDetailsPref(base.ModelBase):
    """Definition model for :obj:`app.bsky.actor.defs`."""

    birth_date: t.Optional[string_formats.DateTime] = None  #: The birth date of account owner.

    py_type: t.Literal['app.bsky.actor.defs#personalDetailsPref'] = Field(
        default='app.bsky.actor.defs#personalDetailsPref', alias='$type', frozen=True
    )


class FeedViewPref(base.ModelBase):
    """Definition model for :obj:`app.bsky.actor.defs`."""

    feed: str  #: The URI of the feed, or an identifier which describes the feed.
    hide_quote_posts: t.Optional[bool] = None  #: Hide quote posts in the feed.
    hide_replies: t.Optional[bool] = None  #: Hide replies in the feed.
    hide_replies_by_like_count: t.Optional[int] = (
        None  #: Hide replies in the feed if they do not have this number of likes.
    )
    hide_replies_by_unfollowed: t.Optional[bool] = True  #: Hide replies in the feed if they are not by followed users.
    hide_reposts: t.Optional[bool] = None  #: Hide reposts in the feed.

    py_type: t.Literal['app.bsky.actor.defs#feedViewPref'] = Field(
        default='app.bsky.actor.defs#feedViewPref', alias='$type', frozen=True
    )


class ThreadViewPref(base.ModelBase):
    """Definition model for :obj:`app.bsky.actor.defs`."""

    prioritize_followed_users: t.Optional[bool] = None  #: Show followed users at the top of all replies.
    sort: t.Optional[
        t.Union[
            t.Literal['oldest'],
            t.Literal['newest'],
            t.Literal['most-likes'],
            t.Literal['random'],
            t.Literal['hotness'],
            str,
        ]
    ] = None  #: Sorting mode for threads.

    py_type: t.Literal['app.bsky.actor.defs#threadViewPref'] = Field(
        default='app.bsky.actor.defs#threadViewPref', alias='$type', frozen=True
    )


class InterestsPref(base.ModelBase):
    """Definition model for :obj:`app.bsky.actor.defs`."""

    tags: t.List[str] = Field(
        max_length=100
    )  #: A list of tags which describe the account owner's interests gathered during onboarding.

    py_type: t.Literal['app.bsky.actor.defs#interestsPref'] = Field(
        default='app.bsky.actor.defs#interestsPref', alias='$type', frozen=True
    )


MutedWordTarget = t.Union[t.Literal['content'], t.Literal['tag'], str]  #: Muted word target


class MutedWord(base.ModelBase):
    """Definition model for :obj:`app.bsky.actor.defs`. A word that the account owner has muted."""

    targets: t.List['models.AppBskyActorDefs.MutedWordTarget']  #: The intended targets of the muted word.
    value: str = Field(max_length=10000)  #: The muted word itself.
    actor_target: t.Optional[t.Union[t.Literal['all'], t.Literal['exclude-following'], str]] = (
        'all'  #: Groups of users to apply the muted word to. If undefined, applies to all users.
    )
    expires_at: t.Optional[string_formats.DateTime] = (
        None  #: The date and time at which the muted word will expire and no longer be applied.
    )
    id: t.Optional[str] = None  #: Id.

    py_type: t.Literal['app.bsky.actor.defs#mutedWord'] = Field(
        default='app.bsky.actor.defs#mutedWord', alias='$type', frozen=True
    )


class MutedWordsPref(base.ModelBase):
    """Definition model for :obj:`app.bsky.actor.defs`."""

    items: t.List['models.AppBskyActorDefs.MutedWord']  #: A list of words the account owner has muted.

    py_type: t.Literal['app.bsky.actor.defs#mutedWordsPref'] = Field(
        default='app.bsky.actor.defs#mutedWordsPref', alias='$type', frozen=True
    )


class HiddenPostsPref(base.ModelBase):
    """Definition model for :obj:`app.bsky.actor.defs`."""

    items: t.List[string_formats.AtUri]  #: A list of URIs of posts the account owner has hidden.

    py_type: t.Literal['app.bsky.actor.defs#hiddenPostsPref'] = Field(
        default='app.bsky.actor.defs#hiddenPostsPref', alias='$type', frozen=True
    )


class LabelersPref(base.ModelBase):
    """Definition model for :obj:`app.bsky.actor.defs`."""

    labelers: t.List['models.AppBskyActorDefs.LabelerPrefItem']  #: Labelers.

    py_type: t.Literal['app.bsky.actor.defs#labelersPref'] = Field(
        default='app.bsky.actor.defs#labelersPref', alias='$type', frozen=True
    )


class LabelerPrefItem(base.ModelBase):
    """Definition model for :obj:`app.bsky.actor.defs`."""

    did: string_formats.Did  #: Did.

    py_type: t.Literal['app.bsky.actor.defs#labelerPrefItem'] = Field(
        default='app.bsky.actor.defs#labelerPrefItem', alias='$type', frozen=True
    )


class BskyAppStatePref(base.ModelBase):
    """Definition model for :obj:`app.bsky.actor.defs`. A grab bag of state that's specific to the bsky.app program. Third-party apps shouldn't use this."""

    active_progress_guide: t.Optional['models.AppBskyActorDefs.BskyAppProgressGuide'] = None  #: Active progress guide.
    nuxs: t.Optional[t.List['models.AppBskyActorDefs.Nux']] = Field(
        default=None, max_length=100
    )  #: Storage for NUXs the user has encountered.
    queued_nudges: t.Optional[t.List[str]] = Field(
        default=None, max_length=1000
    )  #: An array of tokens which identify nudges (modals, popups, tours, highlight dots) that should be shown to the user.

    py_type: t.Literal['app.bsky.actor.defs#bskyAppStatePref'] = Field(
        default='app.bsky.actor.defs#bskyAppStatePref', alias='$type', frozen=True
    )


class BskyAppProgressGuide(base.ModelBase):
    """Definition model for :obj:`app.bsky.actor.defs`. If set, an active progress guide. Once completed, can be set to undefined. Should have unspecced fields tracking progress."""

    guide: str = Field(max_length=100)  #: Guide.

    py_type: t.Literal['app.bsky.actor.defs#bskyAppProgressGuide'] = Field(
        default='app.bsky.actor.defs#bskyAppProgressGuide', alias='$type', frozen=True
    )


class Nux(base.ModelBase):
    """Definition model for :obj:`app.bsky.actor.defs`. A new user experiences (NUX) storage object."""

    completed: bool = False  #: Completed.
    id: str = Field(max_length=100)  #: Id.
    data: t.Optional[str] = Field(
        default=None, max_length=3000
    )  #: Arbitrary data for the NUX. The structure is defined by the NUX itself. Limited to 300 characters.
    expires_at: t.Optional[string_formats.DateTime] = (
        None  #: The date and time at which the NUX will expire and should be considered completed.
    )

    py_type: t.Literal['app.bsky.actor.defs#nux'] = Field(default='app.bsky.actor.defs#nux', alias='$type', frozen=True)


class VerificationPrefs(base.ModelBase):
    """Definition model for :obj:`app.bsky.actor.defs`. Preferences for how verified accounts appear in the app."""

    hide_badges: t.Optional[bool] = False  #: Hide the blue check badges for verified accounts and trusted verifiers.

    py_type: t.Literal['app.bsky.actor.defs#verificationPrefs'] = Field(
        default='app.bsky.actor.defs#verificationPrefs', alias='$type', frozen=True
    )


class PostInteractionSettingsPref(base.ModelBase):
    """Definition model for :obj:`app.bsky.actor.defs`. Default post interaction settings for the account. These values should be applied as default values when creating new posts. These refs should mirror the threadgate and postgate records exactly."""

    postgate_embedding_rules: t.Optional[
        t.List[te.Annotated[t.Union['models.AppBskyFeedPostgate.DisableRule'], Field(discriminator='py_type')]]
    ] = Field(
        default=None, max_length=5
    )  #: Matches postgate record. List of rules defining who can embed this users posts. If value is an empty array or is undefined, no particular rules apply and anyone can embed.
    threadgate_allow_rules: t.Optional[
        t.List[
            te.Annotated[
                t.Union[
                    'models.AppBskyFeedThreadgate.MentionRule',
                    'models.AppBskyFeedThreadgate.FollowerRule',
                    'models.AppBskyFeedThreadgate.FollowingRule',
                    'models.AppBskyFeedThreadgate.ListRule',
                ],
                Field(discriminator='py_type'),
            ]
        ]
    ] = Field(
        default=None, max_length=5
    )  #: Matches threadgate record. List of rules defining who can reply to this users posts. If value is an empty array, no one can reply. If value is undefined, anyone can reply.

    py_type: t.Literal['app.bsky.actor.defs#postInteractionSettingsPref'] = Field(
        default='app.bsky.actor.defs#postInteractionSettingsPref', alias='$type', frozen=True
    )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`tools.ozone.moderation.getRecords`."""

    uris: t.List[string_formats.AtUri] = Field(max_length=100)  #: Uris.


class ParamsDict(t.TypedDict):
    uris: t.List[string_formats.AtUri]  #: Uris.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`tools.ozone.moderation.getRecords`."""

    records: t.List[
        te.Annotated[
            t.Union[
                'models.ToolsOzoneModerationDefs.RecordViewDetail', 'models.ToolsOzoneModerationDefs.RecordViewNotFound'
            ],
            Field(discriminator='py_type'),
        ]
    ]  #: Records.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
    from atproto_client.models.blob_ref import BlobRef
from atproto_client.models import base


class Record(base.RecordModelBase):
    """Record model for :obj:`app.bsky.actor.profile`."""

    avatar: t.Optional['BlobRef'] = (
        None  #: Small image to be displayed next to posts from account. AKA, 'profile picture'.
    )
    banner: t.Optional['BlobRef'] = None  #: Larger horizontal image to display behind profile view.
    created_at: t.Optional[string_formats.DateTime] = None  #: Created at.
    description: t.Optional[str] = Field(default=None, max_length=2560)  #: Free-form profile description text.
    display_name: t.Optional[str] = Field(default=None, max_length=640)  #: Display name.
    joined_via_starter_pack: t.Optional['models.ComAtprotoRepoStrongRef.Main'] = None  #: Joined via starter pack.
    labels: t.Optional[
        te.Annotated[t.Union['models.ComAtprotoLabelDefs.SelfLabels'], Field(default=None, discriminator='py_type')]
    ] = None  #: Self-label values, specific to the Bluesky application, on the overall account.
    pinned_post: t.Optional['models.ComAtprotoRepoStrongRef.Main'] = None  #: Pinned post.

    py_type: t.Literal['app.bsky.actor.profile'] = Field(default='app.bsky.actor.profile', alias='$type', frozen=True)


class GetRecordResponse(base.SugarResponseModelBase):
    """Get record response for :obj:`models.AppBskyActorProfile.Record`."""

    uri: str  #: The URI of the record.
    value: 'models.AppBskyActorProfile.Record'  #: The record.
    cid: t.Optional[str] = None  #: The CID of the record.


class ListRecordsResponse(base.SugarResponseModelBase):
    """List records response for :obj:`models.AppBskyActorProfile.Record`."""

    records: t.Dict[str, 'models.AppBskyActorProfile.Record']  #: Map of URIs to records.
    cursor: t.Optional[str] = None  #: Next page cursor.


class CreateRecordResponse(base.SugarResponseModelBase):
    """Create record response for :obj:`models.AppBskyActorProfile.Record`."""

    uri: str  #: The URI of the record.
    cid: str  #: The CID of the record.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`app.bsky.unspecced.searchPostsSkeleton`."""

    q: str  #: Search query string; syntax, phrase, boolean, and faceting is unspecified, but Lucene query syntax is recommended.
    author: t.Optional[string_formats.AtIdentifier] = (
        None  #: Filter to posts by the given account. Handles are resolved to DID before query-time.
    )
    cursor: t.Optional[str] = (
        None  #: Optional pagination mechanism; may not necessarily allow scrolling through entire result set.
    )
    domain: t.Optional[str] = (
        None  #: Filter to posts with URLs (facet links or embeds) linking to the given domain (hostname). Server may apply hostname normalization.
    )
    lang: t.Optional[string_formats.Language] = (
        None  #: Filter to posts in the given language. Expected to be based on post language field, though server may override language detection.
    )
    limit: t.Optional[int] = Field(default=25, ge=1, le=100)  #: Limit.
    mentions: t.Optional[string_formats.AtIdentifier] = (
        None  #: Filter to posts which mention the given account. Handles are resolved to DID before query-time. Only matches rich-text facet mentions.
    )
    since: t.Optional[str] = (
        None  #: Filter results for posts after the indicated datetime (inclusive). Expected to use 'sortAt' timestamp, which may not match 'createdAt'. Can be a datetime, or just an ISO date (YYYY-MM-DD).
    )
    sort: t.Optional[t.Union[t.Literal['top'], t.Literal['latest'], str]] = (
        'latest'  #: Specifies the ranking order of results.
    )
    tag: t.Optional[t.List[str]] = (
        None  #: Filter to posts with the given tag (hashtag), based on rich-text facet or tag field. Do not include the hash (#) prefix. Multiple tags can be specified, with 'AND' matching.
    )
    until: t.Optional[str] = (
        None  #: Filter results for posts before the indicated datetime (not inclusive). Expected to use 'sortAt' timestamp, which may not match 'createdAt'. Can be a datetime, or just an ISO date (YYY-MM-DD).
    )
    url: t.Optional[string_formats.Uri] = (
        None  #: Filter to posts with links (facet links or embeds) pointing to this URL. Server may apply URL normalization or fuzzy matching.
    )
    viewer: t.Optional[string_formats.Did] = (
        None  #: DID of the account making the request (not included for public/unauthenticated queries). Used for 'from:me' queries.
    )


class ParamsDict(t.TypedDict):
    q: str  #: Search query string; syntax, phrase, boolean, and faceting is unspecified, but Lucene query syntax is recommended.
    author: te.NotRequired[
        t.Optional[string_formats.AtIdentifier]
    ]  #: Filter to posts by the given account. Handles are resolved to DID before query-time.
    cursor: te.NotRequired[
        t.Optional[str]
    ]  #: Optional pagination mechanism; may not necessarily allow scrolling through entire result set.
    domain: te.NotRequired[
        t.Optional[str]
    ]  #: Filter to posts with URLs (facet links or embeds) linking to the given domain (hostname). Server may apply hostname normalization.
    lang: te.NotRequired[
        t.Optional[string_formats.Language]
    ]  #: Filter to posts in the given language. Expected to be based on post language field, though server may override language detection.
    limit: te.NotRequired[t.Optional[int]]  #: Limit.
    mentions: te.NotRequired[
        t.Optional[string_formats.AtIdentifier]
    ]  #: Filter to posts which mention the given account. Handles are resolved to DID before query-time. Only matches rich-text facet mentions.
    since: te.NotRequired[
        t.Optional[str]
    ]  #: Filter results for posts after the indicated datetime (inclusive). Expected to use 'sortAt' timestamp, which may not match 'createdAt'. Can be a datetime, or just an ISO date (YYYY-MM-DD).
    sort: te.NotRequired[
        t.Optional[t.Union[t.Literal['top'], t.Literal['latest'], str]]
    ]  #: Specifies the ranking order of results.
    tag: te.NotRequired[
        t.Optional[t.List[str]]
    ]  #: Filter to posts with the given tag (hashtag), based on rich-text facet or tag field. Do not include the hash (#) prefix. Multiple tags can be specified, with 'AND' matching.
    until: te.NotRequired[
        t.Optional[str]
    ]  #: Filter results for posts before the indicated datetime (not inclusive). Expected to use 'sortAt' timestamp, which may not match 'createdAt'. Can be a datetime, or just an ISO date (YYY-MM-DD).
    url: te.NotRequired[
        t.Optional[string_formats.Uri]
    ]  #: Filter to posts with links (facet links or embeds) pointing to this URL. Server may apply URL normalization or fuzzy matching.
    viewer: te.NotRequired[
        t.Optional[string_formats.Did]
    ]  #: DID of the account making the request (not included for public/unauthenticated queries). Used for 'from:me' queries.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`app.bsky.unspecced.searchPostsSkeleton`."""

    posts: t.List['models.AppBskyUnspeccedDefs.SkeletonSearchPost']  #: Posts.
    cursor: t.Optional[str] = None  #: Cursor.
    hits_total: t.Optional[int] = (
        None  #: Count of search hits. Optional, may be rounded/truncated, and may not be possible to paginate through all hits.
    )



import asyncio
import inspect
import typing as t
from dataclasses import dataclass
from enum import Enum

import typing_extensions as te
from atproto_core.did_doc import DidDocument, is_valid_did_doc
from atproto_server.auth.jwt import get_jwt_payload

if t.TYPE_CHECKING:
    from atproto_server.auth.jwt import JwtPayload

    from atproto_client import models


_SESSION_STRING_SEPARATOR: t.Final[te.Literal[':::']] = ':::'


class SessionEvent(Enum):
    IMPORT = 'import'
    CREATE = 'create'
    REFRESH = 'refresh'


SessionResponse: te.TypeAlias = t.Union[
    'models.ComAtprotoServerCreateSession.Response',
    'models.ComAtprotoServerRefreshSession.Response',
    'Session',
]

SessionChangeCallback = t.Callable[[SessionEvent, 'Session'], None]
AsyncSessionChangeCallback = t.Callable[[SessionEvent, 'Session'], t.Coroutine[t.Any, t.Any, None]]


def _session_exists(session: t.Optional['Session']) -> te.TypeGuard['Session']:
    if not session:
        raise ValueError('Session does not exists. It is not possible to dispatch session change event.')

    return isinstance(session, Session)


class SessionDispatcher:
    def __init__(self, session: t.Optional['Session'] = None) -> None:
        self._session: t.Optional['Session'] = session

        self._on_session_change_callbacks: t.List[SessionChangeCallback] = []
        self._on_session_change_async_callbacks: t.List[AsyncSessionChangeCallback] = []

    def set_session(self, session: 'Session') -> None:
        self._session = session

    def on_session_change(self, callback: t.Union['AsyncSessionChangeCallback', 'SessionChangeCallback']) -> None:
        if inspect.iscoroutinefunction(callback):
            self._on_session_change_async_callbacks.append(callback)
        elif inspect.isfunction(callback):
            self._on_session_change_callbacks.append(callback)

    def dispatch_session_change(self, event: SessionEvent) -> None:
        self._call_on_session_change_callbacks(event)

    async def dispatch_session_change_async(self, event: SessionEvent) -> None:
        self._call_on_session_change_callbacks(event)  # Allow synchronous callbacks in the async client
        await self._call_on_session_change_callbacks_async(event)

    def _call_on_session_change_callbacks(self, event: SessionEvent) -> None:
        assert _session_exists(self._session)
        session_copy = self._session.copy()  # Avoid modifying the original session

        for on_session_change_callback in self._on_session_change_callbacks:
            on_session_change_callback(event, session_copy)

    async def _call_on_session_change_callbacks_async(self, event: SessionEvent) -> None:
        assert _session_exists(self._session)
        session_copy = self._session.copy()  # Avoid modifying the original session

        coroutines: t.List[t.Coroutine[t.Any, t.Any, None]] = []
        for on_session_change_async_callback in self._on_session_change_async_callbacks:
            coroutines.append(on_session_change_async_callback(event, session_copy))

        await asyncio.gather(*coroutines)


@dataclass
class Session:
    handle: str
    did: str
    access_jwt: str
    refresh_jwt: str
    pds_endpoint: t.Optional[str] = 'https://bsky.social'  # Backward compatibility for old sessions

    @property
    def access_jwt_payload(self) -> 'JwtPayload':
        return get_jwt_payload(self.access_jwt)

    @property
    def refresh_jwt_payload(self) -> 'JwtPayload':
        return get_jwt_payload(self.refresh_jwt)

    def __repr__(self) -> str:
        return f'<Session handle={self.handle} did={self.did}>'

    def __str__(self) -> str:
        return self.encode()

    def encode(self) -> str:
        payload = [
            self.handle,
            self.did,
            self.access_jwt,
            self.refresh_jwt,
            self.pds_endpoint,
        ]
        return _SESSION_STRING_SEPARATOR.join(payload)

    @classmethod
    def decode(cls, session_string: str) -> 'Session':
        fields = session_string.split(_SESSION_STRING_SEPARATOR)

        if len(fields) == 4:
            # Old session format
            handle, did, access_jwt, refresh_jwt = fields
            return cls(handle, did, access_jwt, refresh_jwt)

        handle, did, access_jwt, refresh_jwt, pds_endpoint = session_string.split(_SESSION_STRING_SEPARATOR)
        return cls(handle, did, access_jwt, refresh_jwt, pds_endpoint)

    def copy(self) -> 'Session':
        return Session(self.handle, self.did, self.access_jwt, self.refresh_jwt, self.pds_endpoint)

    #: Alias for :attr:`encode`
    export = encode


def get_session_pds_endpoint(session: 'SessionResponse') -> t.Optional[str]:
    """Return the PDS endpoint of the given session.

    Note:
        Return :obj:`None` for self-hosted PDSs.
    """
    if isinstance(session, Session):
        return session.pds_endpoint

    if session.did_doc and is_valid_did_doc(session.did_doc):
        doc = DidDocument.from_dict(session.did_doc)
        return doc.get_pds_endpoint()

    return None



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Label(base.ModelBase):
    """Definition model for :obj:`com.atproto.label.defs`. Metadata tag on an atproto resource (eg, repo or record)."""

    cts: string_formats.DateTime  #: Timestamp when this label was created.
    src: string_formats.Did  #: DID of the actor who created this label.
    uri: (
        string_formats.Uri
    )  #: AT URI of the record, repository (account), or other resource that this label applies to.
    val: str = Field(max_length=128)  #: The short string name of the value or type of this label.
    cid: t.Optional[string_formats.Cid] = (
        None  #: Optionally, CID specifying the specific version of 'uri' resource this label applies to.
    )
    exp: t.Optional[string_formats.DateTime] = None  #: Timestamp at which this label expires (no longer applies).
    neg: t.Optional[bool] = None  #: If true, this is a negation label, overwriting a previous label.
    sig: t.Optional[t.Union[str, bytes]] = None  #: Signature of dag-cbor encoded label.
    ver: t.Optional[int] = None  #: The AT Protocol version of the label object.

    py_type: t.Literal['com.atproto.label.defs#label'] = Field(
        default='com.atproto.label.defs#label', alias='$type', frozen=True
    )


class SelfLabels(base.ModelBase):
    """Definition model for :obj:`com.atproto.label.defs`. Metadata tags on an atproto record, published by the author within the record."""

    values: t.List['models.ComAtprotoLabelDefs.SelfLabel'] = Field(max_length=10)  #: Values.

    py_type: t.Literal['com.atproto.label.defs#selfLabels'] = Field(
        default='com.atproto.label.defs#selfLabels', alias='$type', frozen=True
    )


class SelfLabel(base.ModelBase):
    """Definition model for :obj:`com.atproto.label.defs`. Metadata tag on an atproto record, published by the author within the record. Note that schemas should use #selfLabels, not #selfLabel."""

    val: str = Field(max_length=128)  #: The short string name of the value or type of this label.

    py_type: t.Literal['com.atproto.label.defs#selfLabel'] = Field(
        default='com.atproto.label.defs#selfLabel', alias='$type', frozen=True
    )


class LabelValueDefinition(base.ModelBase):
    """Definition model for :obj:`com.atproto.label.defs`. Declares a label value and its expected interpretations and behaviors."""

    blurs: t.Union[
        t.Literal['content'], t.Literal['media'], t.Literal['none'], str
    ]  #: What should this label hide in the UI, if applied? 'content' hides all of the target; 'media' hides the images/video/audio; 'none' hides nothing.
    identifier: str = Field(
        max_length=100
    )  #: The value of the label being defined. Must only include lowercase ascii and the '-' character ([a-z-]+).
    locales: t.List['models.ComAtprotoLabelDefs.LabelValueDefinitionStrings']  #: Locales.
    severity: t.Union[
        t.Literal['inform'], t.Literal['alert'], t.Literal['none'], str
    ]  #: How should a client visually convey this label? 'inform' means neutral and informational; 'alert' means negative and warning; 'none' means show nothing.
    adult_only: t.Optional[bool] = (
        None  #: Does the user need to have adult content enabled in order to configure this label?
    )
    default_setting: t.Optional[t.Union[t.Literal['ignore'], t.Literal['warn'], t.Literal['hide'], str]] = (
        'warn'  #: The default setting for this label.
    )

    py_type: t.Literal['com.atproto.label.defs#labelValueDefinition'] = Field(
        default='com.atproto.label.defs#labelValueDefinition', alias='$type', frozen=True
    )


class LabelValueDefinitionStrings(base.ModelBase):
    """Definition model for :obj:`com.atproto.label.defs`. Strings which describe the label in the UI, localized into a specific language."""

    description: str = Field(
        max_length=100000
    )  #: A longer description of what the label means and why it might be applied.
    lang: string_formats.Language  #: The code of the language these strings are written in.
    name: str = Field(max_length=640)  #: A short human-readable name for the label.

    py_type: t.Literal['com.atproto.label.defs#labelValueDefinitionStrings'] = Field(
        default='com.atproto.label.defs#labelValueDefinitionStrings', alias='$type', frozen=True
    )


LabelValue = t.Union[
    t.Literal['!hide'],
    t.Literal['!no-promote'],
    t.Literal['!warn'],
    t.Literal['!no-unauthenticated'],
    t.Literal['dmca-violation'],
    t.Literal['doxxing'],
    t.Literal['porn'],
    t.Literal['sexual'],
    t.Literal['nudity'],
    t.Literal['nsfl'],
    t.Literal['gore'],
    str,
]  #: Label value



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_core.cid import CIDType

    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`com.atproto.sync.subscribeRepos`."""

    cursor: t.Optional[int] = None  #: The last known event seq number to backfill from.


class ParamsDict(t.TypedDict):
    cursor: te.NotRequired[t.Optional[int]]  #: The last known event seq number to backfill from.


class Commit(base.ModelBase):
    """Definition model for :obj:`com.atproto.sync.subscribeRepos`. Represents an update of repository state. Note that empty commits are allowed, which include no repo data changes, but an update to rev and signature."""

    blobs: t.List[
        'CIDType'
    ]  #: Blobs. DEPRECATED -- will soon always be empty. List of new blobs (by CID) referenced by records in this commit.
    blocks: t.Union[
        str, bytes
    ]  #: CAR file containing relevant blocks, as a diff since the previous repo state. The commit must be included as a block, and the commit block CID must be the first entry in the CAR header 'roots' list.
    commit: 'CIDType'  #: Repo commit object CID.
    ops: t.List['models.ComAtprotoSyncSubscribeRepos.RepoOp'] = Field(
        max_length=200
    )  #: Ops. List of repo mutation operations in this commit (eg, records created, updated, or deleted).
    rebase: bool  #: DEPRECATED -- unused.
    repo: (
        string_formats.Did
    )  #: The repo this event comes from. Note that all other message types name this field 'did'.
    rev: string_formats.Tid  #: The rev of the emitted commit. Note that this information is also in the commit object included in blocks, unless this is a tooBig event.
    seq: int  #: The stream sequence number of this message.
    time: string_formats.DateTime  #: Timestamp of when this message was originally broadcast.
    too_big: bool  #: DEPRECATED -- replaced by #sync event and data limits. Indicates that this commit contained too many ops, or data size was too large. Consumers will need to make a separate request to get missing data.
    prev_data: t.Optional['CIDType'] = (
        None  #: The root CID of the MST tree for the previous commit from this repo (indicated by the 'since' revision field in this message). Corresponds to the 'data' field in the repo commit object. NOTE: this field is effectively required for the 'inductive' version of firehose.
    )
    since: t.Optional[string_formats.Tid] = None  #: The rev of the last emitted commit from this repo (if any).

    py_type: t.Literal['com.atproto.sync.subscribeRepos#commit'] = Field(
        default='com.atproto.sync.subscribeRepos#commit', alias='$type', frozen=True
    )


class Sync(base.ModelBase):
    """Definition model for :obj:`com.atproto.sync.subscribeRepos`. Updates the repo to a new state, without necessarily including that state on the firehose. Used to recover from broken commit streams, data loss incidents, or in situations where upstream host does not know recent state of the repository."""

    blocks: t.Union[
        str, bytes
    ]  #: CAR file containing the commit, as a block. The CAR header must include the commit block CID as the first 'root'.
    did: string_formats.Did  #: The account this repo event corresponds to. Must match that in the commit object.
    rev: str  #: The rev of the commit. This value must match that in the commit object.
    seq: int  #: The stream sequence number of this message.
    time: string_formats.DateTime  #: Timestamp of when this message was originally broadcast.

    py_type: t.Literal['com.atproto.sync.subscribeRepos#sync'] = Field(
        default='com.atproto.sync.subscribeRepos#sync', alias='$type', frozen=True
    )


class Identity(base.ModelBase):
    """Definition model for :obj:`com.atproto.sync.subscribeRepos`. Represents a change to an account's identity. Could be an updated handle, signing key, or pds hosting endpoint. Serves as a prod to all downstream services to refresh their identity cache."""

    did: string_formats.Did  #: Did.
    seq: int  #: Seq.
    time: string_formats.DateTime  #: Time.
    handle: t.Optional[string_formats.Handle] = (
        None  #: The current handle for the account, or 'handle.invalid' if validation fails. This field is optional, might have been validated or passed-through from an upstream source. Semantics and behaviors for PDS vs Relay may evolve in the future; see atproto specs for more details.
    )

    py_type: t.Literal['com.atproto.sync.subscribeRepos#identity'] = Field(
        default='com.atproto.sync.subscribeRepos#identity', alias='$type', frozen=True
    )


class Account(base.ModelBase):
    """Definition model for :obj:`com.atproto.sync.subscribeRepos`. Represents a change to an account's status on a host (eg, PDS or Relay). The semantics of this event are that the status is at the host which emitted the event, not necessarily that at the currently active PDS. Eg, a Relay takedown would emit a takedown with active=false, even if the PDS is still active."""

    active: (
        bool  #: Indicates that the account has a repository which can be fetched from the host that emitted this event.
    )
    did: string_formats.Did  #: Did.
    seq: int  #: Seq.
    time: string_formats.DateTime  #: Time.
    status: t.Optional[
        t.Union[
            t.Literal['takendown'],
            t.Literal['suspended'],
            t.Literal['deleted'],
            t.Literal['deactivated'],
            t.Literal['desynchronized'],
            t.Literal['throttled'],
            str,
        ]
    ] = None  #: If active=false, this optional field indicates a reason for why the account is not active.

    py_type: t.Literal['com.atproto.sync.subscribeRepos#account'] = Field(
        default='com.atproto.sync.subscribeRepos#account', alias='$type', frozen=True
    )


class Info(base.ModelBase):
    """Definition model for :obj:`com.atproto.sync.subscribeRepos`."""

    name: t.Union[t.Literal['OutdatedCursor'], str]  #: Name.
    message: t.Optional[str] = None  #: Message.

    py_type: t.Literal['com.atproto.sync.subscribeRepos#info'] = Field(
        default='com.atproto.sync.subscribeRepos#info', alias='$type', frozen=True
    )


class RepoOp(base.ModelBase):
    """Definition model for :obj:`com.atproto.sync.subscribeRepos`. A repo operation, ie a mutation of a single record."""

    action: t.Union[t.Literal['create'], t.Literal['update'], t.Literal['delete'], str]  #: Action.
    path: str  #: Path.
    cid: t.Optional['CIDType'] = None  #: For creates and updates, the new record CID. For deletions, null.
    prev: t.Optional['CIDType'] = (
        None  #: For updates and deletes, the previous record CID (required for inductive firehose). For creations, field should not be defined.
    )

    py_type: t.Literal['com.atproto.sync.subscribeRepos#repoOp'] = Field(
        default='com.atproto.sync.subscribeRepos#repoOp', alias='$type', frozen=True
    )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2024 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

from atproto_client.models import string_formats

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Data(base.DataModelBase):
    """Input data model for :obj:`com.atproto.server.createInviteCodes`."""

    code_count: int = 1  #: Code count.
    use_count: int  #: Use count.
    for_accounts: t.Optional[t.List[string_formats.Did]] = None  #: For accounts.


class DataDict(t.TypedDict):
    code_count: int  #: Code count.
    use_count: int  #: Use count.
    for_accounts: te.NotRequired[t.Optional[t.List[string_formats.Did]]]  #: For accounts.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`com.atproto.server.createInviteCodes`."""

    codes: t.List['models.ComAtprotoServerCreateInviteCodes.AccountCodes']  #: Codes.


class AccountCodes(base.ModelBase):
    """Definition model for :obj:`com.atproto.server.createInviteCodes`."""

    account: str  #: Account.
    codes: t.List[str]  #: Codes.

    py_type: t.Literal['com.atproto.server.createInviteCodes#accountCodes'] = Field(
        default='com.atproto.server.createInviteCodes#accountCodes', alias='$type', frozen=True
    )
